@@ -10,6 +10,7 @@ from datetime import datetime
 import logging
 
 from blinker import signal
+from pymongo import DESCENDING
 
 from .settings import settings
 from .helpers import MockCollection
@@ -36,15 +37,14 @@ class MongoModel(object):
     post_delete = signal('post_delete')
 
     def __init__(self, *args, **kwargs):
-        if '_id' in kwargs:
-            self._id = kwargs['_id']
+        self.__dict__.update(kwargs)
 
     def to_bson(self):
-        raise NotImplementedError
+        return self.__dict__
 
     def save(self):
         if hasattr(self, '_id'):
-            previous = self.c.find_one(dict(_id=self._id))
+            previous = self.get(_id=self._id)
         else:
             previous = None
         self.pre_save.send(previous)
@@ -114,7 +114,6 @@ class MongoModel(object):
 class VersioningMongoModel(MongoModel):
 
     def __init__(self, *args, **kwargs):
-        self._previous = None
         super(VersioningMongoModel, self).__init__(*args, **kwargs)
 
     @classmethod
@@ -125,33 +124,34 @@ class VersioningMongoModel(MongoModel):
         # Need to verify that the data contained in self is actually still in
         # the collection
         if hasattr(self, '_id'):
-            curr = self.c.find_one(dict(_id=self._id))
+            curr = self.get(_id=self._id)
+            curr = curr.to_bson() if curr else None
         else:
             curr = None
-        prev = self._previous
+        prev = self.previous_version().to_bson()
         # Both models are present so something's changed between them
         if prev and curr:
             current_fields = curr.keys()
             previous_fields = prev.keys()
             added = [f for f in current_fields if f not in previous_fields]
             deleted = [f for f in previous_fields if f not in current_fields]
-            is_equal = lambda f: f in previous_fields and prev[f] == curr[f]
-            changed = [f for f in current_fields if not is_equal(f)]
+            changed = [f for f in current_fields if (f in previous_fields
+                                                     and prev[f] != curr[f])]
             delta = dict(
                 added=dict([(k, curr[k]) for k in added]),
-                deleted=deleted,
-                changed=dict([(k, curr[k]) for k in changed]),
+                deleted=dict([(k, prev[k]) for k in deleted]),
+                changed=dict([(k, (prev[k], curr[k])) for k in changed]),
             )
         elif prev:  # Model's been deleted
             delta = dict(
                 added={},
-                deleted=prev.keys(),
+                deleted=prev,
                 changed={},
             )
         elif curr:  # Model's been added
             delta = dict(
                 added=curr,
-                deleted=[],
+                deleted={},
                 changed={},
             )
         else:       # Neither model exists
@@ -165,11 +165,41 @@ class VersioningMongoModel(MongoModel):
             doc = dict(
                 switch_id=self._id,
                 timestamp=datetime.utcnow(),
-                delta=self._diff(),
+                delta=delta,
                 **kwargs
             )
             self._versioned_collection().save(doc)
-        self._previous = self.to_bson()
+
+    def _unpack_delta(self, version):
+        '''
+        Helper function that makes it easier to access the data nested with a
+        delta. Returns a tuple of (delta, added, deleted, changed).
+        '''
+        delta = version.get('delta', {})
+        added = delta.get('added', {})
+        deleted = delta.get('deleted', {})
+        changed = delta.get('changed', {})
+        return delta, added, deleted, changed
+
+    def previous_version(self):
+        if not hasattr(self, '_id'):
+            return self.__class__()
+        vc = self._versioned_collection()
+        versions = vc.find(dict(switch_id=self._id))
+        previous = dict()
+        # build up the previous state based on all past deltas
+        if versions:
+            versions = versions.sort('timestamp')
+            for v in versions:
+                delta, added, deleted, changed = self._unpack_delta(v)
+                previous.update(added)
+                for k in deleted.keys():
+                    if k in previous:
+                        del previous[k]
+                for k, v in changed.iteritems():
+                    old, new = v
+                    previous[k] = new
+        return self.__class__(**previous)
 
 
 class Switch(VersioningMongoModel):
@@ -365,7 +395,8 @@ class Switch(VersioningMongoModel):
 
         return self.STATUS_LABELS[status]
 
-    # TODO: Consolidate to_bson and to_dict; they should be the same.
+    # TODO: Consolidate to_bson and to_dict; they should be the same. It should
+    # be as simple as spitting out __dict__.
     def to_dict(self, manager):
         data = {
             'key': self.key,
@@ -397,16 +428,12 @@ class Switch(VersioningMongoModel):
             data['conditions'].append(last)
         return data
 
-    def to_bson(self):
-        data = dict(
-            key=self.key,
-            status=self.status,
-            label=self.label,
-            description=self.description,
-            date_created=self.date_created,
-            date_modified=self.date_modified,
-            value=self.value
-        )
-        if hasattr(self, '_id'):
-            data['_id'] = self._id
-        return data
+    def list_versions(self):
+        '''
+        Return a display-friendly list of all versions.
+        '''
+        vc = self._versioned_collection()
+        versions = vc.find(dict(switch_id=self._id))
+        if not versions:
+            return dict(versions={})
+        return versions.sort('timestamp', DESCENDING)
